// This file is part of WADBD
// https://github.com/rhythmcache 
// https://rhythmcache.t.me 

const MODULE_PATH = '/data/adb/modules/wadbd';
let callbackCounter = 0;
const themes = ['light', 'dark', 'amoled'];
let currentThemeIndex = localStorage.getItem('themeIndex') ? parseInt(localStorage.getItem('themeIndex')) : 0;
let isEnvironmentSupported = true;

let showNotifications = localStorage.getItem('showNotifications') === 'true';

async function fetchWADBDVersion() {
    const versionElement = document.getElementById('wadbdVersion');

    if (!versionElement) {
        console.error('Element with id "wadbdVersion" not found');
        return;
    }

    try {
        const { stdout: moduleProp } = await exec('cat /data/adb/modules/wadbd/module.prop | grep version= | sed s/version=//g');
        versionElement.textContent = moduleProp.trim();
    } catch (error) {
        console.error('Failed to fetch WADBD version:', error);
        versionElement.textContent = 'Error fetching version';
    }
}

function toggleInfoDialog() {
    const dialog = document.getElementById('infoDialog');
    if (dialog.style.display === 'none') {
        dialog.style.display = 'flex';
        fetchWADBDVersion();
    } else {
        dialog.style.display = 'none';
    }
}
function toggleSettingsMenu() {
    const settingsMenu = document.getElementById('settingsMenu');
    settingsMenu.style.display = settingsMenu.style.display === 'none' ? 'flex' : 'none';
}

document.addEventListener('click', function(event) {
    const settingsMenu = document.getElementById('settingsMenu');
    const settingsButton = document.querySelector('.settings-button');
    if (!settingsMenu.contains(event.target) && !settingsButton.contains(event.target)) {
        settingsMenu.style.display = 'none';
    }
});
function openLink(url) {
    if (typeof ksu !== 'undefined' && ksu.exec) {
        ksu.exec(`am start -a android.intent.action.VIEW -d "${url}"`, '{}', 'openLink_callback');
    } else {
        window.open(url, '_blank');
    }
}

function setTheme(index) {
    const theme = themes[index];
    document.body.setAttribute('data-theme', theme === 'dark' ? '' : theme);
    localStorage.setItem('themeIndex', index);
}

function toggleTheme() {
    currentThemeIndex = (currentThemeIndex + 1) % themes.length;
    setTheme(currentThemeIndex);
}

function getUniqueCallbackName(base) {
    return `${base}_${Date.now()}_${callbackCounter++}`;
}

function alert(message) {
    if (typeof ksu !== 'undefined' && ksu.toast) {
        ksu.toast(message);
    } else {
        console.log(`Toast: ${message}`);
    }
}
async function exec(command) {
    return new Promise((resolve, reject) => {
        const callbackName = getUniqueCallbackName('exec');
        window[callbackName] = (errno, stdout, stderr) => {
            resolve({ errno, stdout: stdout.trim(), stderr });
            delete window[callbackName];
        };
        try {
            ksu.exec(command, '{}', callbackName);
        } catch (error) {
            reject(error);
            delete window[callbackName];
        }
    });
}
async function initializeEnvironment() {
    try {
        if (typeof ksu === 'undefined' && typeof mmrl === 'undefined' && typeof $wadbd === 'undefined') {
            isEnvironmentSupported = false;
            showUnsupportedDialog();
            return;
        }

        if (typeof ksu === 'undefined' || !ksu.exec) {
            isEnvironmentSupported = false;

            if (typeof mmrl !== 'undefined' && typeof $wadbd !== 'undefined') {
                const ksuDialog = document.getElementById('ksuDialog');
                ksuDialog.style.display = 'flex';
                const requestKsuApiButton = document.getElementById('requestKsuApiButton');
                requestKsuApiButton.addEventListener('click', async () => {
                    try {
                        await $wadbd.requestAdvancedKernelSUAPI();
                        alert("KernelSU API access requested. Please grant access.");
                        ksuDialog.style.display = 'none';
                        initializeEnvironment();
                    } catch (error) {
                        console.log("Error requesting KernelSU API:", error);
                        alert("Failed to request KernelSU API access.");
                    }
                });
            }
            return;
        }

        const { errno } = await exec('id');
        isEnvironmentSupported = errno === 0;
    } catch {
        isEnvironmentSupported = false;
    }

    updateStatus();
    updateDeviceInfo();
}
function showUnsupportedDialog() {
    const unsupportedDialog = document.getElementById('unsupportedDialog');
    unsupportedDialog.style.display = 'flex';
}

async function updateDeviceInfo() {
    const deviceInfoDiv = document.getElementById('deviceInfo');
    
    if (!isEnvironmentSupported) {
        deviceInfoDiv.innerHTML = '<div class="error-message">Kernel SU API is missing. Unsupported Environment</div>';
        return;
    }

    try {
        const [
            { stdout: deviceCodename },
            { stdout: deviceName },
            { stdout: androidVersion },
            { stdout: securityPatch },
            { stdout: kernelVersion }
        ] = await Promise.all([
            exec('getprop ro.product.device'),
            exec('getprop ro.product.model'),
            exec('getprop ro.build.version.release'),
            exec('getprop ro.build.version.security_patch'),
            exec('uname -r')
        ]);

        deviceInfoDiv.innerHTML = `
            <div class="info-item">
                <span class="info-label">Device Codename :</span>
                <span class="info-value">${deviceCodename || 'Unknown'}</span>
            </div>
            <div class="info-item">
                <span class="info-label">Device Name :</span>
                <span class="info-value">${deviceName || 'Unknown'}</span>
            </div>
            <div class="info-item">
                <span class="info-label">Android Version :</span>
                <span class="info-value">${androidVersion || 'Unknown'}</span>
            </div>
            <div class="info-item">
                <span class="info-label">Security Patch :</span>
                <span class="info-value">${securityPatch || 'Unknown'}</span>
            </div>
            <div class="info-item">
                <span class="info-label">Kernel Version :</span>
                <span class="info-value">${kernelVersion || 'Unknown'}</span>
            </div>
        `;
    } catch (error) {
        deviceInfoDiv.innerHTML = '<div class="error-message">Error fetching device information</div>';
    }
}

document.addEventListener("DOMContentLoaded", function () {
    const wirelessToggle = document.getElementById("wirelessToggle");
    const portInput = document.getElementById("portInput");
    const changePortBtn = document.getElementById("changePortBtn");

    portInput.addEventListener("input", function () {
        if (this.value.length > 5) {
            this.value = this.value.slice(0, 5);
        }
    });

    function isValidPort(port) {
        return port === "" || (/^\d{1,5}$/.test(port) && port >= 1025 && port <= 65535);
    }

    wirelessToggle.addEventListener("change", function () {
        const port = portInput.value.trim();

        if (this.checked) {
            if (!isValidPort(port)) {
                alert("❌ Invalid port! Enter a port between 1024 and 65535.");
                this.checked = false; 
            }
        }
    });
    changePortBtn.addEventListener("click", function () {
        const port = portInput.value.trim();
        if (!isValidPort(port)) {
            alert("❌ Invalid port! Enter a port between 1024 and 65535.");
        } else {
            alert(`✅ Port changed to: ${port || "5555"}`);
        }
    });
});

async function getBootPort() {
    try {
        const { stdout: serviceContent } = await exec(`cat "${MODULE_PATH}/service.sh"`);
        const portMatch = serviceContent.match(/setprop service\.adb\.tcp\.port (\d+)/);
        return portMatch ? portMatch[1] : null;
    } catch {
        return null;
    }
}

async function updateStatus() {
    const statusDiv = document.getElementById('statusOutput');
    
    if (!isEnvironmentSupported) {
        statusDiv.innerHTML = '<div class="error-message">Kernel SU API is missing. Unsupported Environment.</div>';
        setAllControlsDisabled(true);
        return;
    }

    try {
        const [
            { stdout: usbDebug },
            { stdout: wirelessPort },
            { stdout: adbdStatus },
            { stdout: adbdPid }
        ] = await Promise.all([
            exec('settings get global adb_enabled'),
            exec('getprop service.adb.tcp.port'),
            exec('getprop init.svc.adbd'),
            exec('pidof adbd')
        ]);

        const bootEnabled = await checkBootStatus();
        const bootPort = await getBootPort();
        const isWirelessEnabled = wirelessPort !== '' && wirelessPort !== '-1';

        statusDiv.innerHTML = `
            <div class="status-item">
                <span class="status-label">USB Debugging : </span>
                <span>${usbDebug === '1' ? 'Enabled' : 'Disabled'}</span>
            </div>
            <div class="status-item">
                <span class="status-label">Wireless ADB : </span>
                <span>${isWirelessEnabled ? `Enabled (Port ${wirelessPort})` : 'Disabled'}</span>
            </div>
            <div class="status-item">
                <span class="status-label">ADB Daemon : </span>
                <span>${adbdStatus || 'Unknown'} ${adbdStatus.trim() === 'running' ? `(PID: ${adbdPid.trim() || 'N/A'})` : ''}</span>
            </div>
            <div class="status-item">
                <span class="status-label">WADBD on Boot : </span>
                <span>${bootEnabled ? `Enabled (Port ${bootPort || 'Unknown'})` : 'Not enabled'}</span>
            </div>
        `;
        document.getElementById('usbDebugToggle').checked = usbDebug === '1';
        document.getElementById('wirelessToggle').checked = isWirelessEnabled;
        document.getElementById('bootToggle').checked = bootEnabled;
        
        setAllControlsDisabled(false);
    } catch (error) {
        statusDiv.innerHTML = '<div class="error-message">Error fetching status</div>';
        setAllControlsDisabled(true);
    }
}


async function toggleUsbDebug(enabled) {
    if (!isEnvironmentSupported) return;
    
    try {
        await exec(`settings put global adb_enabled ${enabled ? '1' : '0'}`);
        updateStatus();
    } catch (error) {
        console.error('Error toggling USB debugging:', error);
    }
}



async function toggleWirelessAdb(enabled) {
    if (!isEnvironmentSupported) return;
    
    try {
        let port = parseInt(document.getElementById('portInput').value) || 5555;
        if (port < 1024 || port > 65535) {
            alert("Invalid port! Please enter a port between 1024 and 65535.");
            return;
        }

        await exec(`setprop service.adb.tcp.port ${enabled ? port : '-1'}`);
        await exec('stop adbd');
        await exec('start adbd');

        if (enabled && showNotifications) {
            await exec(`su -lp 2000 -c "cmd notification post -S bigtext -t 'WADBD' 'Tag' 'Wireless ADBD Enabled on port ${port} !!'" > /dev/null 2>&1`);
        }

        updateStatus();
    } catch (error) {
        console.error('Error toggling wireless ADB:', error);
    }
}

async function checkBootStatus() {
    try {
        const { stdout } = await exec(`[ -f "${MODULE_PATH}/service.sh" ] && echo "1" || echo "0"`);
        if (stdout === '1') {
            const { stdout: content } = await exec(`cat "${MODULE_PATH}/service.sh"`);
            return content.includes('setprop service.adb.tcp.port');
        }
        return false;
    } catch {
        return false;
    }
}

async function toggleBoot(enabled) {
    if (!isEnvironmentSupported) return;
    
    try {
        if (enabled) {
            let port = parseInt(document.getElementById('portInput').value) || 5555;
            if (port < 1024 || port > 65535) {
                alert("Invalid port! Please enter a port between 1024 and 65535.");
                return;
            }

            let script = `#!/system/bin/sh
while [ "$(getprop sys.boot_completed)" != "1" ]; do
    sleep 2
done
setprop service.adb.tcp.port ${port}
stop adbd
start adbd`;

            if (showNotifications) {
                script += `\nsu -lp 2000 -c "cmd notification post -S bigtext -t \\"WADBD\\" \\"Tag\\" \\"Wireless ADBD Enabled on port ${port} !!\\"" > /dev/null 2>&1`;
            }

            await exec(`mkdir -p "${MODULE_PATH}"`);
            await exec(`echo '${script}' > "${MODULE_PATH}/service.sh"`);
            await exec(`chmod 755 "${MODULE_PATH}/service.sh"`);
        } else {
            await exec(`rm -f "${MODULE_PATH}/service.sh"`);
        }
        updateStatus();
    } catch (error) {
        console.error('Error toggling boot status:', error);
    }
}

function setAllControlsDisabled(disabled) {
    const controls = [
        'usbDebugToggle',
        'wirelessToggle',
        'bootToggle',
        'portInput',
        'changePortBtn'
    ];
    
    controls.forEach(id => {
        const element = document.getElementById(id);
        if (element) {
            element.disabled = disabled;
        }
    });
}



document.getElementById('usbDebugToggle').addEventListener('change', (e) => {
    toggleUsbDebug(e.target.checked);
});

document.getElementById('wirelessToggle').addEventListener('change', (e) => {
    toggleWirelessAdb(e.target.checked);
});

document.getElementById('bootToggle').addEventListener('change', (e) => {
    toggleBoot(e.target.checked);
});

document.getElementById('changePortBtn').addEventListener('click', async () => {
    const wirelessEnabled = document.getElementById('wirelessToggle').checked;
    const bootEnabled = document.getElementById('bootToggle').checked;
    
    if (wirelessEnabled) {
        await toggleWirelessAdb(true);
    }
    if (bootEnabled) {
        await toggleBoot(true);
    }
});

async function updateConnectionInfo() {
    const connectionInfoDiv = document.getElementById('connectionInfo');
    
    if (!isEnvironmentSupported) {
        connectionInfoDiv.innerHTML = '<div class="error-message">Kernel SU API is missing. Unsupported Environment</div>';
        return;
    }

    try {
        const { stdout: wirelessPort } = await exec('getprop service.adb.tcp.port');
        const isWirelessEnabled = wirelessPort !== '' && wirelessPort !== '-1';
        
        if (!isWirelessEnabled) {
            connectionInfoDiv.innerHTML = `
                <div class="info-message">
                    Wireless ADB is currently disabled. Enable it to see connection commands.
                </div>
            `;
            return;
        }

        const { stdout: ipInfo } = await exec('ip -f inet addr show');
        
        const ipMatches = [...ipInfo.matchAll(/inet (\d+\.\d+\.\d+\.\d+).*(?:wlan|rmnet|eth)/g)];
        
        if (ipMatches.length === 0) {
            connectionInfoDiv.innerHTML = `
                <div class="warning-message">
                    No valid network interfaces found. Make sure you're connected to a network.
                </div>
            `;
            return;
        }

        let html = `
            <div class="connection-section">
                <div class="connection-header">Available Connection Commands:</div>
                <div class="network-interfaces">`;

        ipMatches.forEach(match => {
            const ip = match[1];
            html += `
                <div class="command-group">
                    <div class="interface-info">Interface IP: ${ip}</div>
                    <div class="command-box">
                        <code>adb connect ${ip}:${wirelessPort}</code>
                        <button class="copy-button" onclick="copyToClipboard('adb connect ${ip}:${wirelessPort}')">Copy</button>
                    </div>
                </div>
            `;
        });

        html += `
                </div>
                <div class="connection-note">
                    Note: Make sure ADB is installed on your computer and both devices are on the same network.
                </div>
            </div>`;

        connectionInfoDiv.innerHTML = html;
    } catch (error) {
        connectionInfoDiv.innerHTML = '<div class="error-message">Error fetching connection information</div>';
    }
}


function copyToClipboard(text) {
    navigator.clipboard.writeText(text).catch(err => {
        console.error('Failed to copy: ', err);
    });
}

window.onload = () => {
    setTheme(currentThemeIndex);
    initializeEnvironment();
    const notificationToggle = document.getElementById('notificationToggle');
    notificationToggle.checked = showNotifications;
};

document.getElementById('notificationToggle').addEventListener('change', (e) => {
    showNotifications = e.target.checked;
    localStorage.setItem('showNotifications', showNotifications);
});

setInterval(() => {
    updateStatus();
    updateConnectionInfo();
}, 2000);
